/*
 *	binkleyforce -- unix FTN mailer project
 *	
 *	Copyright (c) 1998-2000 Alexander Belkin, 2:5020/1398.11
 *	
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *	
 *	$Id: outb_flo.c,v 1.3 2000/05/26 20:04:27 adb Exp $
 */

#include "includes.h"
#include "confread.h"
#include "logger.h"
#include "util.h"
#include "outbound.h"

/*
 * Return "TRUE" if no more files to send left in FLO
 */
bool out_flo_isempty(FILE *fp)
{
	char s[BF_MAXPATH+1], *p;;
	
	if( fseek(fp, 0, SEEK_SET) ) return FALSE;
	
	while( fgets(s, sizeof(s), fp) )
	{
		string_chomp(s);
		if( *s == '@' || *s == '^' || *s == '#' )
			p = s+1;
		else
			p = s;
		if( *s != '~' && access(p, F_OK) == 0 ) return FALSE;
	}
	
	return TRUE;
}

/*
 * Mark the file name in FLO file as sent ( with '~' prefix )
 */
int out_flo_marksent(const char *fname, const char *floname)
{
	FILE *fp;
	char s[BF_MAXPATH+1], *p;
	size_t strsize  = 0;
	size_t last_off = 0;
	
	if( (fp = fopen(floname, "r+")) == NULL )
		{ logerr("can't open flo file \"%s\"", floname); return 1; }
	
	while( fgets(s, sizeof(s), fp) )
	{
		strsize = strlen(s);
		p = s;
		
		string_chomp(s);
		
		if( *p == '@' || *p == '^' || *p == '#' ) ++p;
		
		if( *s != '~' && strcmp(p, fname) == 0 )
		{
			if( fseek(fp, (long)last_off, SEEK_SET) == 0 )
			{
				if( fputc('~', fp) == EOF )
					logerr("can't mark file as sent in \"%s\"", floname);
			}
			break;
		}
		last_off += strsize;
	}
	fclose(fp);
	return 0;
}

int out_flo_getsizes(const char *floname, size_t *arcmail, size_t *files)
{
	FILE *fp;
	char s[BF_MAXPATH+1], *p;
	struct stat st;
    
	ASSERT(floname != NULL && arcmail != NULL && files != NULL);
	
	if( (fp=fopen(floname, "r")) == NULL )
		{ logerr("can't open flo \"%s\"", floname); return 1; }
    
	while( fgets(s, sizeof(s), fp) )
	{
		string_chomp(s);
		if( *s && *s != '~' )
		{
			if( *s == '-' || *s == '^' || *s == '#' || *s == '@' )
				p  = s + 1;
			else
				p = s;

			if( stat(p, &st) == 0 )
			{
				if( out_filetype(p) & TYPE_ARCMAIL )
					*arcmail += st.st_size;
				else
					*files += st.st_size;
			}
		}
	}
	fclose(fp);
	return 0;
}

int out_flo_unlinkempty(const s_flofile *flotab, int flonum)
{
	int i;
	FILE *fp;
	bool isempty;
	
	for( i = 0; i < flonum; i++ )
	{
		if( (fp = fopen(flotab[i].fname, "r")) )
		{
			isempty = out_flo_isempty(fp);
			fclose(fp);
			if( isempty && unlink(flotab[i].fname) == -1 )
				logerr("cannot unlink empty flo file \"%s\"", flotab[i].fname);
		}
	}
	return 0;
}
